<?php
namespace Teknisa\Model\Gestaopessoas\Repositories;

use HCM\Util\Repositories;
use HCM\Util\Util;

class FpaMsgcalculo extends \Doctrine\ORM\EntityRepository {
    
    public function limpaMsgCalculoVinculo($nrcalculofolha, $nrvinculom, $nrorg, $dsorigmsg)
    {
        $fpa_msgcalculo = Repositories::FPA_MSGCALCULO;
        $sql = <<<SQL
DELETE FROM $fpa_msgcalculo fm
 WHERE (fm.nrcalculofolha = :nrcalculofolha OR (fm.nrcalculofolha IS NULL AND :nrcalculofolha IS NULL))
   AND fm.nrvinculom = :nrvinculom
   AND fm.nrorg = :nrorg
SQL;
        $params = array(
            "nrcalculofolha" => $nrcalculofolha,
            "nrvinculom" => $nrvinculom,
            "nrorg" => $nrorg
        );
        if ($dsorigmsg) {
            $sql .= " AND fm.dsorigmsg = :dsorigmsg";
            $params['dsorigmsg'] = $dsorigmsg;
        }

        $result = $this->getEntityManager()->createQuery($sql)->execute($params);
    }

    public function limpaMsgCalculoVinculoCompetencia($nrcalculofolha, $nrvinculom, $nrorg, $dt_inicio, $dt_fim, $dsorigmsg){
        $fpa_msgcalculo = Repositories::FPA_MSGCALCULO;
        $sql = <<<SQL
DELETE FROM $fpa_msgcalculo fm
 WHERE fm.nrcalculofolha = :nrcalculofolha 
   AND fm.nrvinculom = :nrvinculom
   AND fm.nrorg = :nrorg
   AND (fm.dtinclusao BETWEEN :dt_inicio AND :dt_fim)
   
SQL;
        $params = array(
            "nrcalculofolha" => $nrcalculofolha,
            "nrvinculom"     => $nrvinculom,
            "nrorg"          => $nrorg,
            "dt_inicio"      => $dt_inicio,
            "dt_fim"         => $dt_fim
        );
        if ($dsorigmsg) {
            $sql .= " AND fm.dsorigmsg = :dsorigmsg";
            $params['dsorigmsg'] = $dsorigmsg;
        }
        $result = $this->getEntityManager()->createQuery($sql)->execute($params);
    }
    
    public function limpaMsgCalculoProcesso($nrcalculofolha, $nrorg, $dsorigmsg, $cdoperador)
    {
    $fpa_msgcalculo = Repositories::FPA_MSGCALCULO;
    

        $sql = <<<SQL
DELETE FROM $fpa_msgcalculo fm
 WHERE fm.nrcalculofolha = :nrcalculofolha
   AND fm.nrorg = :nrorg
   AND fm.dsorigmsg = :dsorigmsg
   AND fm.cdoperinclusao = :cdoperador
SQL;

        $params = array(
            "nrcalculofolha" => $nrcalculofolha,
            "nrorg" => $nrorg,
            "dsorigmsg" => $dsorigmsg,
            "cdoperador" => $cdoperador 
        );

        $result = $this->getEntityManager()->createQuery($sql)->execute($params);
        
    }
    
    public function limpaMsgCalculo($nrcalculofolha, $nrorg, $dsorigmsg){
        $fpa_msgcalculo = Repositories::FPA_MSGCALCULO;
    

        $sql = <<<SQL
DELETE FROM $fpa_msgcalculo fm
 WHERE fm.nrcalculofolha = :nrcalculofolha
   AND fm.nrorg = :nrorg
   AND fm.dsorigmsg = :dsorigmsg
SQL;

        $params = array(
            "nrcalculofolha" => $nrcalculofolha,
            "nrorg" => $nrorg,
            "dsorigmsg" => $dsorigmsg
        );

        $result = $this->getEntityManager()->createQuery($sql)->execute($params);
    }
    
    public function getMsgFiltrada($nrcalculobenef, $dsOriMsg, $nrorg, $dtmescompetenc, $dsOrdem, $ds_inicio, $ds_fim){
        
        $fpa_msgcalculo = Repositories::FPA_MSGCALCULO;
        $gpe_vinculom   = Repositories::GPE_VINCULOM;       
        $gpe_vinculoh   = Repositories::GPE_VINCULOH;       
       
        $vinculoInicio = $this->getEntityManager()->getRepository(Repositories::GPE_VINCULOM)->findOneBy(
            [
                "nrorg"      => $nrorg,
                "nrvinculom" => $ds_inicio,
            ]
        );

        $vinculoFim = $this->getEntityManager()->getRepository(Repositories::GPE_VINCULOM)->findOneBy(
            [
                "nrorg"      => $nrorg,
                "nrvinculom" => $ds_fim,
            ]
        );
        
        $condIntervalo  = Util::getIntervalCondition($dsOrdem, $vinculoInicio->getNmvinculom(), $vinculoFim->getNmvinculom());
        

        $params = 
        [
            'p_nrorg'          => $nrorg, 
            'p_dtmescompetenc' => $dtmescompetenc,
            'p_ds_inicio'      => $vinculoInicio->getNmvinculom(),
            'p_ds_fim'         => $vinculoFim->getNmvinculom(),
            'p_nrcalcfolha'    => $nrcalculobenef,
            'p_dsorigmsg'      => $dsOriMsg,
        ];   
        
$sql = <<<SQL
    SELECT
        msgc
        
    FROM $fpa_msgcalculo msgc
    
    INNER JOIN $gpe_vinculom vm 
        WITH msgc.nrorg      = vm.nrorg
        AND  msgc.nrvinculom = vm.nrvinculom
        
     JOIN $gpe_vinculoh vh 
        WITH vh.nrorg = vm.nrorg
        AND vh.nrvinculom = vm.nrvinculom
        AND vh.dtmescompetenc = (
                                    SELECT MAX(vh2.dtmescompetenc)
    
                                    FROM $gpe_vinculoh vh2
    
                                    WHERE vh2.nrvinculom = vh.nrvinculom
                                        AND vh2.nrorg = vh.nrorg
                                        AND vh2.dtmescompetenc <= :p_dtmescompetenc
                                )

    WHERE msgc.nrorg = :p_nrorg
        AND msgc.nrcalculofolha = :p_nrcalcfolha
        AND msgc.dsorigmsg      = :p_dsorigmsg
        AND $condIntervalo        
SQL;

        $retorno = $this->getEntityManager()->createQuery($sql)->execute($params);
        return $retorno;
    }
}